@@ -43,9 +43,12 @@ pub enum Instruction {
 
     DecodeForeign(Target, usize, Arc<ForeignType>, Vec<usize>),
     DecodeRef(Target, usize, String, Vec<usize>),
-    DecodeEnum(String, PrimitiveType, usize, Target),
+    DecodeRepr(String, PrimitiveType, usize, Target),
     DecodePrimitive(Target, usize, PrimitiveType),
     DecodePrimitiveArray(Target, usize, PrimitiveType, Option<usize>),
+    DecodeReprArray(Target, usize, String, PrimitiveType, Option<usize>),
+    // target, register of length
+    Skip(Target, usize),
 
     // register representing: internal stream, end index, terminator, output handle, inner
     Loop(
@@ -68,6 +71,7 @@ pub struct Context {
     pub register_count: usize,
     pub field_register_map: HashMap<String, usize>,
     pub instructions: Vec<Instruction>,
+    pub name: String,
 }
 
 impl Context {
@@ -81,6 +85,7 @@ impl Context {
 impl Context {
     pub fn new() -> Context {
         Context {
+            name: String::new(),
             instructions: vec![],
             field_register_map: HashMap::new(),
             register_count: 0,
@@ -89,11 +94,13 @@ impl Context {
 
     pub fn decode_field_top(&mut self, field: &Arc<Field>) {
         assert!(field.toplevel);
+        self.name = field.name.clone();
         let mut value = self.decode_field(Target::Direct, field);
         match &*field.type_.borrow() {
             Type::Foreign(_) => (),
             Type::Container(_) => (),
             Type::Enum(_) => (),
+            Type::Bitfield(_) => (),
             _ => {
                 if let Some(old_value) = value {
                     let extra_value = self.alloc_register();
@@ -222,15 +229,17 @@ impl Context {
                             Type::Container(c) => {
                                 let mut values = vec![];
                                 for (subname, subchild) in c.flatten_view() {
-                                    if !matches!(&*subchild.type_.borrow(), Type::Container(_)) {
-                                        values.push((
-                                            subname.clone(),
-                                            *self
-                                                .field_register_map
-                                                .get(&subname)
-                                                .expect("missing field in field_register_map"),
-                                        ));
+                                    if subchild.is_pad.get() || matches!(&*subchild.type_.borrow(), Type::Container(_)) {
+                                        continue;
                                     }
+    
+                                    values.push((
+                                        subname.clone(),
+                                        *self
+                                            .field_register_map
+                                            .get(&subname)
+                                            .expect("missing field in field_register_map"),
+                                    ));
                                 }
 
                                 self.instructions.push(Instruction::Construct(target, Constructable::TaggedEnumStruct {
@@ -273,15 +282,16 @@ impl Context {
                         let emitted = self.alloc_register();
                         let mut items = vec![];
                         for (name, child) in c.flatten_view() {
-                            if !matches!(&*child.type_.borrow(), Type::Container(_)) {
-                                items.push((
-                                    name.clone(),
-                                    *self
-                                        .field_register_map
-                                        .get(&name)
-                                        .expect("missing field in field_register_map"),
-                                ));
+                            if child.is_pad.get() || matches!(&*child.type_.borrow(), Type::Container(_)) {
+                                continue;
                             }
+                            items.push((
+                                name.clone(),
+                                *self
+                                    .field_register_map
+                                    .get(&name)
+                                    .expect("missing field in field_register_map"),
+                            ));
                         }
                         self.instructions.push(Instruction::Construct(
                             emitted,
@@ -296,17 +306,29 @@ impl Context {
                     }
                 }
             }
-            _ => Some(self.decode_type(source, field)),
+            _ => self.decode_type(source, field),
         };
 
         emitted
     }
 
-    pub fn decode_type(&mut self, source: Target, field: &Arc<Field>) -> usize {
+    pub fn decode_type(&mut self, source: Target, field: &Arc<Field>) -> Option<usize> {
         let output = self.alloc_register();
-        match &*field.type_.borrow() {
+        Some(match &*field.type_.borrow() {
             Type::Container(_) => unimplemented!(),
             Type::Array(c) => {
+                if field.is_pad.get() {
+                    let array_type = field.type_.borrow();
+                    let array_type = match &*array_type {
+                        Type::Array(a) => &**a,
+                        _ => panic!("invalid type for pad"),
+                    };
+                    let len = array_type.length.value.as_ref().cloned().unwrap();
+                    let length_register = self.alloc_register();
+                    self.instructions.push(Instruction::Eval(length_register, len));
+                    self.instructions.push(Instruction::Skip(source, length_register));
+                    return None;
+                }
                 let terminator = if c.length.expandable && c.length.value.is_some() {
                     let len = c.length.value.as_ref().cloned().unwrap();
                     let r = self.alloc_register();
@@ -329,17 +351,30 @@ impl Context {
                     && c.element.transforms.borrow().len() == 0
                     && terminator.is_none()
                 {
-                    match &*c.element.type_.borrow() {
+                    let type_ = c.element.type_.borrow();
+                    let type_ = type_.resolved();
+                    match &*type_ {
                         // todo: const-length type optimizations for container/array/foreign
                         Type::Container(_) | Type::Array(_) | Type::Foreign(_) | Type::Ref(_) => (),
                         Type::Enum(x) => {
-                            self.instructions.push(Instruction::DecodePrimitiveArray(
+                            self.instructions.push(Instruction::DecodeReprArray(
                                 source,
                                 output,
-                                PrimitiveType::Scalar(x.rep),
+                                field.name.clone(),
+                                PrimitiveType::Scalar(x.rep.clone()),
                                 len,
                             ));
-                            return output;
+                            return Some(output);
+                        }
+                        Type::Bitfield(x) => {
+                            self.instructions.push(Instruction::DecodeReprArray(
+                                source,
+                                output,
+                                field.name.clone(),
+                                PrimitiveType::Scalar(x.rep.clone()),
+                                len,
+                            ));
+                            return Some(output);
                         }
                         Type::Scalar(x) => {
                             self.instructions.push(Instruction::DecodePrimitiveArray(
@@ -348,7 +383,7 @@ impl Context {
                                 PrimitiveType::Scalar(*x),
                                 len,
                             ));
-                            return output;
+                            return Some(output);
                         }
                         Type::F32 => {
                             self.instructions.push(Instruction::DecodePrimitiveArray(
@@ -357,7 +392,7 @@ impl Context {
                                 PrimitiveType::F32,
                                 len,
                             ));
-                            return output;
+                            return Some(output);
                         }
                         Type::F64 => {
                             self.instructions.push(Instruction::DecodePrimitiveArray(
@@ -366,7 +401,7 @@ impl Context {
                                 PrimitiveType::F64,
                                 len,
                             ));
-                            return output;
+                            return Some(output);
                         }
                         Type::Bool => {
                             self.instructions.push(Instruction::DecodePrimitiveArray(
@@ -375,7 +410,7 @@ impl Context {
                                 PrimitiveType::Bool,
                                 len,
                             ));
-                            return output;
+                            return Some(output);
                         }
                     }
                 }
@@ -393,7 +428,16 @@ impl Context {
                 output
             }
             Type::Enum(e) => {
-                self.instructions.push(Instruction::DecodeEnum(
+                self.instructions.push(Instruction::DecodeRepr(
+                    field.name.clone(),
+                    PrimitiveType::Scalar(e.rep.clone()),
+                    output,
+                    source,
+                ));
+                output
+            }
+            Type::Bitfield(e) => {
+                self.instructions.push(Instruction::DecodeRepr(
                     field.name.clone(),
                     PrimitiveType::Scalar(e.rep.clone()),
                     output,
@@ -466,6 +510,6 @@ impl Context {
                 }
                 output
             }
-        }
+        })
     }
 }
