@@ -14,6 +14,7 @@ fn emit_target(target: &Target) -> TokenStream {
 }
 
 fn prepare_decode(
+    options: &CompileOptions,
     context: &Context,
     instructions: &[Instruction],
     is_async: bool,
@@ -34,6 +35,12 @@ fn prepare_decode(
     }
 
     for instruction in instructions.iter() {
+        if options.debug_mode {
+            let raw = format!("decode {}: {:?}", context.name, instruction);
+            statements.push(quote! {
+                println!("{}", #raw);
+            });
+        }
         match instruction {
             Instruction::Eval(target, expr) => {
                 let target = emit_register(*target);
@@ -166,7 +173,7 @@ fn prepare_decode(
                 let transformed = transformer
                     .inner
                     .decoding_gen(input.clone(), args, is_async);
-                let prelude = prepare_decode(context, &prelude[..], is_async, false);
+                let prelude = prepare_decode(options, context, &prelude[..], is_async, false);
 
                 //todo: would be nicer to use generics here instead of trait object
                 if is_async {
@@ -228,19 +235,18 @@ fn prepare_decode(
                     });
                 }
             }
-            Instruction::DecodeEnum(name, type_, value, target) => {
+            Instruction::DecodeRepr(name, type_, value, target) => {
                 let target = emit_target(target);
                 let value = emit_register(*value);
 
                 let enum_ident = format_ident!("{}", &name);
-                let rep = format_ident!("{}", type_.to_string());
                 let length = type_.size() as usize;
 
                 statements.push(quote! {
                     let #value = {
                         let mut scratch = [0u8; #length];
                         #target.read_exact(&mut scratch[..])#async_?;
-                        #enum_ident::from_repr(#rep::from_be_bytes((&scratch[..]).try_into()?))?
+                        #enum_ident::from_repr(#type_::from_be_bytes((&scratch[..]).try_into()?))?
                     };
                 });
             }
@@ -277,33 +283,54 @@ fn prepare_decode(
                     statements.push(quote! {
                         let #data = {
                             let t_count = #len as usize;
-                            let mut t: Vec<#type_> = Vec::with_capacity(t_count);
-                            unsafe { t.set_len(t_count); }
-                            let t_borrow = &mut t[..];
-                            let t_borrow2 = unsafe {
-                                let len = t_borrow.len() * mem::size_of::<#type_>();
-                                let ptr = t.as_ptr() as *mut u8;
-                                slice::from_raw_parts_mut(ptr, len)
-                            };
-                            #target.read_exact(&mut t_borrow2[..])#async_?;
-                            t
+                            let size = mem::size_of::<#type_>();
+                            let mut raw: Vec<u8> = Vec::with_capacity(t_count * size);
+                            unsafe { raw.set_len(t_count * size) };
+                            #target.read_exact(&mut raw[..])#async_?;
+                            raw.chunks_exact(size).map(|x| #type_::from_be_bytes(x.try_into().unwrap())).collect()
                         };
                     });
                 } else {
                     statements.push(quote! {
                         let #data = {
-                            let mut t: Vec<u8> = Vec::new();
-                            #target.read_to_end(&mut t)#async_?;
-                            let t = Box::leak(t.into_boxed_slice());
-                            let size = t.len() / mem::size_of::<#type_>();
-                            unsafe { Vec::<#type_>::from_raw_parts(t.as_mut_ptr() as *mut #type_, size, size) }
+                            let mut raw: Vec<u8> = Vec::new();
+                            #target.read_to_end(&mut raw)#async_?;
+                            let size = mem::size_of::<#type_>();
+                            raw.chunks_exact(size).map(|x| #type_::from_be_bytes(x.try_into().unwrap())).collect()
+                        };
+                    });
+                }
+            }
+            Instruction::DecodeReprArray(target, data, name, type_, len) => {
+                let target = emit_target(target);
+                let data = emit_register(*data);
+                let enum_ident = format_ident!("{}", &name);
+                if let Some(len) = len {
+                    let len = emit_register(*len);
+                    statements.push(quote! {
+                        let #data = {
+                            let t_count = #len as usize;
+                            let size = mem::size_of::<#type_>();
+                            let mut raw: Vec<u8> = Vec::with_capacity(t_count * size);
+                            unsafe { raw.set_len(t_count * size) };
+                            #target.read_exact(&mut raw[..])#async_?;
+                            raw.chunks_exact(size).map(|x| #enum_ident::from_repr(#type_::from_be_bytes(x.try_into().unwrap()))).collect()
+                        };
+                    });
+                } else {
+                    statements.push(quote! {
+                        let #data = {
+                            let mut raw: Vec<u8> = Vec::new();
+                            #target.read_to_end(&mut raw)#async_?;
+                            let size = mem::size_of::<#type_>();
+                            raw.chunks_exact(size).map(|x| #enum_ident::from_repr(#type_::from_be_bytes(x.try_into().unwrap()))).collect()
                         };
                     });
                 }
             }
             Instruction::Loop(target, stop_index, terminator, output, inner) => {
                 let output = emit_register(*output);
-                let inner = prepare_decode(context, &inner[..], is_async, false);
+                let inner = prepare_decode(options, context, &inner[..], is_async, false);
                 let stop = stop_index.map(emit_register);
                 let terminator = terminator.map(emit_register);
                 let target = emit_target(target);
@@ -344,8 +371,8 @@ fn prepare_decode(
 
                             {
                                 let mut #target = Cursor::new(r);
-                                let #target = &mut reader;
-                                while reader.position() < r_len {
+                                let #target = &mut #target;
+                                while #target.position() < r_len {
                                     #inner
                                 }
                             }
@@ -364,7 +391,7 @@ fn prepare_decode(
                 let target = emit_register(*target);
                 let interior = emit_register(*interior);
                 let condition = emit_register(*condition);
-                let inner = prepare_decode(context, &inner[..], is_async, false);
+                let inner = prepare_decode(options, context, &inner[..], is_async, false);
                 statements.push(quote! {
                     let #target = if #condition {
                         #inner
@@ -376,7 +403,7 @@ fn prepare_decode(
             }
             Instruction::ConditionalPredicate(condition, inner) => {
                 let condition = emit_register(*condition);
-                let inner = prepare_decode(context, &inner[..], is_async, false);
+                let inner = prepare_decode(options, context, &inner[..], is_async, false);
                 statements.push(quote! {
                     if #condition {
                         #inner
@@ -391,7 +418,15 @@ fn prepare_decode(
             },
             Instruction::Error(e) => {
                 statements.push(quote! {
-                    return Err(DecodeError(#e.to_string()).into());
+                    return Err(decode_error(#e).into());
+                });
+            },
+            Instruction::Skip(target, len) => {
+                let target = emit_target(target);
+                let len = emit_register(*len);
+                statements.push(quote! {
+                    let mut big_scratch = vec![0u8; #len as usize];
+                    #target.read_exact(&mut big_scratch[..])#async_?;
                 });
             },
         }
@@ -403,8 +438,8 @@ fn prepare_decode(
     }
 }
 
-pub fn prepare_decoder(coder: &Context, is_async: bool) -> TokenStream {
-    let decode = prepare_decode(&coder, &coder.instructions[..], is_async, true);
+pub fn prepare_decoder(options: &CompileOptions, coder: &Context, is_async: bool) -> TokenStream {
+    let decode = prepare_decode(options, &coder, &coder.instructions[..], is_async, true);
     quote! {
         #decode
     }
