@@ -44,6 +44,7 @@ pub enum Token {
     LeftCurly,
     RightCurly,
     Enum,
+    Bitfield,
     LtEq,
     GtEq,
     Eq,
@@ -118,6 +119,7 @@ impl fmt::Display for Token {
             LeftCurly => write!(f, "{{"),
             RightCurly => write!(f, "}}"),
             Enum => write!(f, "enum "),
+            Bitfield => write!(f, "bitfield "),
             LtEq => write!(f, "<= "),
             GtEq => write!(f, ">= "),
             Eq => write!(f, "== "),
@@ -170,7 +172,7 @@ fn eat_identifier(input: &[u8]) -> Option<(&[u8], &[u8])> {
     }
     let mut i = 1usize;
     while i < input.len() {
-        if !input[i].is_ascii_alphanumeric() && input[i] != b'_' && input[i] != b'-' {
+        if !input[i].is_ascii_alphanumeric() && input[i] != b'_' {
             break;
         }
         i += 1;
@@ -410,6 +412,7 @@ impl Token {
                     "f32" => Token::F32,
                     "f64" => Token::F64,
                     "enum" => Token::Enum,
+                    "bitfield" => Token::Bitfield,
                     "bool" => Token::Bool,
                     "from" => Token::From,
                     "true" => Token::True,
@@ -507,7 +510,7 @@ impl fmt::Debug for SpannedToken {
     }
 }
 
-pub fn tokenize(input: &str) -> Result<Vec<SpannedToken>> {
+pub fn tokenize(input: &str, strip_comments: bool) -> Result<Vec<SpannedToken>> {
     let mut input = input.as_bytes();
     let mut tokens = vec![];
     let mut index = 064;
@@ -516,10 +519,20 @@ pub fn tokenize(input: &str) -> Result<Vec<SpannedToken>> {
     while input.len() > 0 {
         match Token::gobble(input) {
             (output, Some(token)) => {
+                let start_line = line_no;
+                match &token {
+                    Token::CommentLine(_) => {
+                        line_no += 1;
+                    },
+                    Token::CommentBlock(s) => {
+                        line_no += s.chars().filter(|x| *x == '\n').count() as u64;
+                    }
+                    _ => (),
+                }
                 tokens.push(SpannedToken {
                     token,
                     span: Span {
-                        line_start: line_no,
+                        line_start: start_line,
                         line_stop: line_no,
                         col_start: index - line_start + 1,
                         col_stop: index - line_start + (input.len() - output.len()) as u64 + 1,
@@ -547,7 +560,11 @@ pub fn tokenize(input: &str) -> Result<Vec<SpannedToken>> {
             }
         }
     }
-    Ok(tokens)
+    if strip_comments {
+        Ok(tokens.into_iter().filter(|x| !matches!(x.token, Token::CommentLine(_) | Token::CommentBlock(_))).collect())
+    } else {
+        Ok(tokens)
+    }
 }
 
 #[cfg(test)]
@@ -558,6 +575,7 @@ mod tests {
     fn test_string() {
         let tokens = tokenize(
             r#""test" "test\"test""#,
+            false,
         )
         .unwrap();
         let mut output = String::new();
@@ -595,6 +613,7 @@ mod tests {
         f32
         f64
         enum
+        bitfield
         true
         false
         bool
@@ -602,6 +621,7 @@ mod tests {
         ,;:?[]{}<>?+-/ *%..<=>= = == != ! () // test$
         :> || && | ^ | >> << >>>~ . ?:
         //"#,
+        false,
         )
         .unwrap();
         let mut output = String::new();
@@ -612,7 +632,7 @@ mod tests {
             output,
             r#"test_ident"string""str"ing""str\ing"12345-12345type as import import_ffi i8 u8 transform function const /*
 
-        test block*/ container f32 f64 enum true false bool from ,;:?[]{}< > ?+-/ *%.. <= >= = == != ! ()// test$
+        test block*/ container f32 f64 enum bitfield true false bool from ,;:?[]{}< > ?+-/ *%.. <= >= = == != ! ()// test$
 :> || && | ^| >> << >>> ~. ?: //
 "#
         );
